"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const api_1 = require("mesg-js/lib/api");
const b58 = tslib_1.__importStar(require("mesg-js/lib/util/base58"));
const encoder_1 = require("mesg-js/lib/util/encoder");
const util_1 = require("util");
const root_command_1 = tslib_1.__importDefault(require("../../root-command"));
class ProcessLogs extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.services = {};
    }
    async run() {
        const { args } = this.parse(ProcessLogs);
        const process = await this.api.process.get({
            hash: b58.decode(args.PROCESS_HASH)
        });
        if (!process.hash) {
            throw new Error('invalid process hash');
        }
        if (process.nodes) {
            const instanceHashes = process.nodes
                .map(({ event, result, task }) => {
                if (event)
                    return event.instanceHash;
                if (result)
                    return result.instanceHash;
                if (task)
                    return task.instanceHash;
            })
                .filter(x => x && x.length > 0);
            for (const hash of instanceHashes) {
                if (!hash)
                    continue;
                const instance = await this.api.instance.get({ hash });
                const service = await this.api.service.get({ hash: instance.serviceHash });
                this.services[b58.encode(hash)] = service;
            }
        }
        const streams = [];
        const results = this.api.execution.stream({
            filter: {
                statuses: [
                    api_1.ExecutionStatus.COMPLETED,
                    api_1.ExecutionStatus.FAILED,
                ],
            }
        });
        streams.push(() => results.cancel());
        results
            .on('data', this.handlerResult(process.hash))
            .on('error', (error) => { this.warn('Result stream error: ' + error.message); });
        return {
            destroy: () => {
                streams.forEach(s => s());
            }
        };
    }
    handlerResult(processHash) {
        return (execution) => {
            if (!execution.processHash)
                return;
            if (b58.encode(execution.processHash) !== b58.encode(processHash))
                return;
            this.log(this.formatResult(execution));
        };
    }
    formatResult(execution) {
        if (!execution.instanceHash)
            return;
        const prefix = [
            `[${execution.stepID}]`,
            b58.encode(execution.instanceHash),
            this.services[b58.encode(execution.instanceHash)].sid,
            execution.taskKey,
        ].join(' - ');
        if (execution.error) {
            return `${prefix}: ` + chalk_1.default.red('ERROR:', execution.error);
        }
        if (!execution.outputs)
            return;
        return prefix +
            '\n\tinputs:  ' + chalk_1.default.gray(util_1.inspect(encoder_1.decode(execution.inputs || {}))) +
            '\n\toutputs: ' + chalk_1.default.gray(util_1.inspect(encoder_1.decode(execution.outputs || {}))) +
            '\n';
    }
}
exports.default = ProcessLogs;
ProcessLogs.description = 'Log the executions related to a process';
ProcessLogs.flags = Object.assign({}, root_command_1.default.flags);
ProcessLogs.args = [{
        name: 'PROCESS_HASH',
        required: true,
    }];
