"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const base58 = tslib_1.__importStar(require("mesg-js/lib/util/base58"));
const credential_command_1 = require("../../credential-command");
const error_1 = require("../../utils/error");
const compile_1 = tslib_1.__importDefault(require("./compile"));
const create_1 = tslib_1.__importDefault(require("./create"));
const logs_1 = tslib_1.__importDefault(require("./logs"));
const start_1 = tslib_1.__importDefault(require("./start"));
const stop_1 = tslib_1.__importDefault(require("./stop"));
class ServiceDev extends credential_command_1.WithCredential {
    constructor() {
        super(...arguments);
        this.instanceCreated = false;
    }
    async run() {
        const { args, flags } = this.parse(ServiceDev);
        this.spinner.start('Starting service');
        this.spinner.status = 'compiling';
        const definition = await compile_1.default.run([args.SERVICE, '--silent']);
        this.spinner.status = 'creating service';
        const serviceHash = await this.createService(definition);
        this.spinner.status = 'starting service';
        const instanceHash = await this.startService(serviceHash, flags.env);
        this.spinner.status = 'fetching logs';
        const stream = await logs_1.default.run([base58.encode(instanceHash)]);
        this.spinner.stop('ready');
        process.once('SIGINT', async () => {
            stream.destroy();
            if (this.instanceCreated)
                await stop_1.default.run([base58.encode(instanceHash)]);
        });
    }
    async createService(definition) {
        const { hash } = await this.api.service.hash(definition);
        if (!hash)
            throw new Error('invalid hash');
        const { exists } = await this.api.service.exists({ hash });
        if (!exists) {
            const service = await this.api.service.create(definition, await this.getCredential());
            if (!service.hash)
                throw new Error('invalid hash');
            if (service.hash.toString() !== hash.toString())
                throw new Error('invalid hash');
        }
        return hash;
    }
    async startService(serviceHash, env) {
        try {
            const instance = await this.api.instance.create({
                serviceHash,
                env
            });
            if (!instance.hash)
                throw new Error('invalid hash');
            this.instanceCreated = true;
            return instance.hash;
        }
        catch (e) {
            if (!error_1.IsAlreadyExistsError.match(e))
                throw e;
            this.warn('service already started');
            return new error_1.IsAlreadyExistsError(e).hash;
        }
    }
}
exports.default = ServiceDev;
ServiceDev.description = 'Run a service in development mode';
ServiceDev.flags = Object.assign(Object.assign(Object.assign({}, credential_command_1.WithCredential.flags), create_1.default.flags), start_1.default.flags);
ServiceDev.args = [{
        name: 'SERVICE',
        description: 'Path or url of a service',
        default: './'
    }];
